package it.danieleverducci.ojo.ui;

import android.content.Context;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import org.videolan.libvlc.IVLCVout;
import org.videolan.libvlc.LibVLC;
import org.videolan.libvlc.Media;
import org.videolan.libvlc.MediaPlayer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import it.danieleverducci.ojo.R;
import it.danieleverducci.ojo.Settings;
import it.danieleverducci.ojo.databinding.FragmentSurveillanceBinding;
import it.danieleverducci.ojo.entities.Camera;

/**
 * Some streams to test:
 * rtsp://wowzaec2demo.streamlock.net/vod/mp4:BigBuckBunny_115k.mov
 * rtsp://demo:demo@ipvmdemo.dyndns.org:5541/onvif-media/media.amp?profile=profile_1_h264&sessiontimeout=60&streamtype=unicast
 */
public class SurveillanceFragment extends Fragment implements MediaPlayer.EventListener, IVLCVout.Callback {

    final static private String[] VLC_OPTIONS = new String[]{
            "--aout=opensles",
            //"--audio-time-stretch", // time stretching
            //"-vvv", // verbosity
            "--avcodec-codec=h264",
            //"--file-logging",
            //"--logfile=vlc-log.txt"
    };

    private FragmentSurveillanceBinding binding;
    private List<CameraView> cameraViews = new ArrayList<>();

    @Override
    public View onCreateView(
            LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState
    ) {

        binding = FragmentSurveillanceBinding.inflate(inflater, container, false);
        return binding.getRoot();

    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        Settings settings = Settings.fromDisk(getContext());
        List<Camera> cc = settings.getCameras();

        int elemsPerSide = calcGridSideElements(cc.size());
        int camIdx = 0;
        for (int r = 0; r < elemsPerSide; r++) {
            // Create row and add to row container
            LinearLayout row = new LinearLayout(getContext());
            LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                    LinearLayout.LayoutParams.MATCH_PARENT,
                    0,
                    1.0f
            );
            binding.gridRowContainer.addView(row, params);
            // Add camera viewers to the row
            for (int c = 0; c < elemsPerSide; c++) {
                if ( camIdx < cc.size() ) {
                    Camera cam = cc.get(camIdx);
                    addCameraView(cam, row);
                } else {
                    // Cameras are less than the maximum number of cells in grid: fill remaining cells with empty views
                    View ev = new View(getContext());
                    ev.setBackgroundColor(Color.BLUE);
                    LinearLayout.LayoutParams evParams = new LinearLayout.LayoutParams(
                            0,
                            LinearLayout.LayoutParams.MATCH_PARENT,
                            1.0f
                    );
                    row.addView(ev, evParams);
                }
                camIdx++;
            }
        }






    }

    @Override
    public void onStart() {
        super.onStart();

        for (CameraView cv : cameraViews) {
            cv.startPlayback();
        }
    }

    @Override
    public void onPause() {
        super.onPause();

        for (CameraView cv : cameraViews) {
            cv.pausePlayback();
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        for (CameraView cv : cameraViews) {
            cv.destroy();
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
        // TODO Release player
    }

    /**
     * VLC media listener
     */
    @Override
    public void onEvent(MediaPlayer.Event event) {
        //TODO
    }

    /**
     * Video output callbacks
     */
    @Override
    public void onSurfacesCreated(IVLCVout vlcVout) {

    }

    @Override
    public void onSurfacesDestroyed(IVLCVout vlcVout) {

    }

    private void addCameraView(Camera camera, LinearLayout rowContainer) {
        CameraView cv = new CameraView(
                getContext(),
                camera,
                this
        );

        // Add to layout
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                0,
                LinearLayout.LayoutParams.MATCH_PARENT,
                1.0f
        );
        rowContainer.addView(cv.surfaceView, params);

        cameraViews.add(cv);
    }

    /**
     * Returns the number of elements per side needed to create a grid that can contain the provided elements number.
     * Es: to display 3 elements is needed a 4-element grid, with 2 elements per side (a 2x2 grid)
     * Es: to display 7 elements is needed a 9-element grid, with 3 elements per side (a 3x3 grid)
     * @param elements
     */
    private int calcGridSideElements(int elements) {
        return (int)(Math.ceil(Math.sqrt(elements)));
    }

    /**
     * Contains all entities (views and java entities) related to a camera stream viewer
     */
    private class CameraView {
        protected SurfaceView surfaceView;
        protected MediaPlayer mediaPlayer;
        protected IVLCVout ivlcVout;
        protected Camera camera;
        protected LibVLC libvlc;
        protected IVLCVout.Callback callback;

        public CameraView(Context context, Camera camera, IVLCVout.Callback callback) {
            this.camera = camera;
            this.callback = callback;
            this.libvlc = new LibVLC(context, new ArrayList<>(Arrays.asList(VLC_OPTIONS)));

            surfaceView = new SurfaceView(context);
            SurfaceHolder holder = surfaceView.getHolder();

            holder.setKeepScreenOn(true);

            // Create media player
            mediaPlayer = new MediaPlayer(libvlc);
//            mediaPlayer.setEventListener(this);

            // Set up video output
            ivlcVout = mediaPlayer.getVLCVout();
            ivlcVout.setVideoView(surfaceView);
            ivlcVout.addCallback(this.callback);
            ivlcVout.attachViews();

            // Load media and start playing
            Media m = new Media(libvlc, Uri.parse(camera.getRtspUrl()));
            mediaPlayer.setMedia(m);

            // Register for view resize events
            final ViewTreeObserver observer= surfaceView.getViewTreeObserver();
            observer.addOnGlobalLayoutListener(() -> {
                // Set rendering size
                ivlcVout.setWindowSize(surfaceView.getWidth(), surfaceView.getHeight());
            });
        }

        /**
         * Starts the playback.
         */
        public void startPlayback() {
            mediaPlayer.play();
        }

        /**
         * Pauses the playback.
         */
        public void pausePlayback() {
            mediaPlayer.pause();
        }

        /**
         * Destroys the object and frees the memory
         */
        public void destroy() {
            mediaPlayer.stop();
            final IVLCVout vout = mediaPlayer.getVLCVout();
            vout.removeCallback(this.callback);
            vout.detachViews();
            libvlc.release();
            libvlc = null;
        }
    }
}